/*---------------------------------------------------------------------------
  This code is part of the book
  LOGIC PROGRAMMING TECHNIQUES: THE PROLOG LANGUAGE
  Ilias Sakellariou, Nikos Bassiliades, Petros Kefalas, Demosthenes Stamatis
  Hellenic Academic On-line Publications, 2015
----------------------------------------------------------------------------*/


%%% Chapter 9, Examples.


%%% list_max_n/2
%%% list_max_n(Max,List)
list_max_n(Max,List):-
 member(Max,List),
 not((member(X,List), X>Max)).


%%% list_max_n/2
%%% list_max_n(Max,List)
list_max_nc(Max,List):-
 member(Max,List),
 not((member(X,List), X>Max)),
 !.


%%% share elements
common_element(X,ListA,ListB):-
 member(X,ListA),
 member(X,ListB).


%%% share elements
common_element_a(X,ListA,ListB):-
 member(X,ListA),
 member(X,ListB),!.

%%% share elements
common_element_b(X,ListA,ListB):-
 member(X,ListA),
 !,
 member(X,ListB).



%%% share elements
common_element_c(X,ListA,ListB):-
 member(X,ListA),
 member_check(X,ListB).

%%% member_check/2
%%% member_check Succeeds once per ground element, 
%%% or "returns" the first element.
member_check(X,[X|_]):-!.
member_check(X,[_|Rest]):-
	member_check(X,Rest).

%%% member_check_a/2
%%% !!INCORRECT version of member_check.
member_check_a(X,[X|_]).
member_check_a(X,[_|Rest]):-
	member_check_a(X,Rest),!.


%%% member_check_b/2
%%% !!INCORRECT version of member_check.
member_check_b(X,[X|_]).
member_check_b(X,[_|Rest]):-
	!,member_check_b(X,Rest).


%%% true member once
member_once(X,List):-
	member_once_aux(X,List,[]).

member_once_aux(X,[X|_],Seen):-
	not(member(X,Seen)).
member_once_aux(X,[Y|Rest],Seen):-
	member_once_aux(X,Rest,[Y|Seen]).


%%% expression_sign/2
%%% expression_sign(Expr, Sign)
%%% Characterises the result of the arithmetic expression.
expression_sign(X,positive):- X > 0,!.
expression_sign(X,zero):- X =:= 0,!.
expression_sign(_,negative).

%%% expression_sign_incorrect/2
%%% expression_sign_incorrect(Expr, Sign)
%%% !!INCORRECT version of expression_sign
expression_sign_incorrect(X,positive):- X > 0.
expression_sign_incorrect(X,zero):- X =:=0.
expression_sign_incorrect(_,negative).


%%% intersect/3
%%% intersect(L1,L2,L3)
%%% Succeeds if L3 is the intersection of lists L1 and L2.
intersect([],_,[]).
intersect([X|Rest],List,[X|IntList]):-
	member(X,List), 
	intersect(Rest,List,IntList).
intersect([X|Rest],List,IntList):-
	not(member(X,List)),	 
	intersect(Rest,List,IntList).


intersect_cut([],_,[]).
intersect_cut([X|Rest],List,[X|IntList]):-
	member(X,List), 
	intersect_cut(Rest,List,IntList).
intersect_cut([X|Rest],List,IntList):-
	not(member(X,List)),	 
	intersect_cut(Rest,List,IntList).

%%% not_member
not_member(X,List):-member(X,List),!,fail.
not_member(_,_).

	
not_member2(_,[]).
not_member2(X,[X|_]):-!,fail.
not_member2(X,[_|Rest]):-
	not_member2(X,Rest).


%%% write_list/1
%%% write_list(List)
%%% Outputs the elements of a List using write/1
%%% and suceeds. The predicate uses recursion.
write_list([]).
write_list([X|Tail]) :-
	write(X), nl,
	write_list(Tail).

%%% write_list_f/1
%%% write_list_f(List)
%%% Outputs the elements of a List using write/1
%%% and suceeds. The predicate uses failure driven loops.
write_list_f(List):-
	member(X,List),
	write(X), nl,
	fail.

write_list_f(_).


%%% print_cartesian_product/2
%%% print_cartesian_product(L1,L2)
print_cartesian_product(L1,L2):-
	member(X,L1),
	member(Y,L2),
	write(X),write("-"),write(Y), nl,
	fail.


print_cartesian_product(_,_).


%%% wait/1
%%% wait(N).
%%% Blocks Execution for N seconds
wait(N):-
  get_time(Start),
  repeat,
  get_time(End),
  End > Start + N,
  !.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% INPUT OUTPUT Predicates
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%% write_for_ever/1
%%% write_for_ever(Text)
%%% Outputs to the console, the text infinite times.
write_for_ever(Text):-
  repeat,
  write(Text),nl,
  fail.

%%% user_input_yn/1
%%% user_input_yn(Ans)
%%% Unifies Ans with the user answer, that must be y or n.

user_input_yn(Ans):-
	repeat,
	write('Please answer y/n '),nl,
	read(Ans),
	(Ans = y ; Ans = n),
	!.


%%% print_to_file/2
%%% print_to_file(FileName,Content)
%%% Writes content to a new file FileName
print_to_file(FileName,Content):-
	tell(FileName),
	write(Content),nl, 
	told. 

%%% print_terms_of_file/1
%%% print_terms_of_file(FileName)
%%% Prints on console the contents of a file using read.

print_terms_of_file(FileName):-	
	see(FileName),
	repeat,
	read(Term),
	handle_file_term(Term),
	!, 
	seen.
	

%%% handle_file_term/1
%%% handle_file_term(Term)
%%% 


handle_file_term(end_of_file):- !.

handle_file_term(Term):-	
	write(Term),nl,
	fail.



%%% Reading Char by Char.
%%% print_chars_of_file(Files)
print_contents_of_file(FileName):-	
	see(FileName),
	repeat,
	get_code(CharCode),
	( 
          (CharCode = -1,!,seen) 
            ; 	
          (put_code(CharCode),fail)
        ).

%%% concatenate_atoms/2
%%% concatenate_atoms(Atom1,Atom2,Atom)
%%% Succeeds if Atom is the concatenation of Atom 1 and Atom2.

concatenate_atoms(Atom1,Atom2,Atom):-
	atom_codes(Atom1,L1),
	atom_codes(Atom2,L2),
	append(L1,L2,L),
	atom_codes(Atom,L).

%%% copy_file/2
%%% copy_file(SourceFile,DestFile)
%%% Copies all the contents of the sourceFile to the (new) DestFile.
	 
copy_file(InFile,OutFile):-
	open(InFile,read,InStream),
	open(OutFile,write,OutStream),
	repeat,
	get_code(InStream,CharCode),
	handle_char_code(OutStream,CharCode),
	!, 
	close(InStream),
	close(OutStream). 

%%% handle_char_code/2
%%% Auxiliary predicate for the copy_file/2.
%%% Handles appropriately characters by 
handle_char_code(_,-1):-!.

handle_char_code(OutStream,CharCode):-
	put_code(OutStream,CharCode),
	fail.
	

%%% Cut Red 
max1(X,Y,X):- X>=Y.
max1(X,Y,Y):- X=<Y. 

max2(X,Y,X):- X>=Y, !.
max2(_,Y,Y).

	

