/*---------------------------------------------------------------------------
  This code is part of the book
  LOGIC PROGRAMMING TECHNIQUES: THE PROLOG LANGUAGE
  Ilias Sakellariou, Nikos Bassiliades, Petros Kefalas, Demosthenes Stamatis
  Hellenic Academic On-line Publications, 2015
----------------------------------------------------------------------------*/


%%% A simple lexer 

read_tokens(File,Tokens):-
	open(File,read,Stream),
	get_code(Stream,LookAhead),
	read_tokens_from(Stream,LookAhead,Tokens),
	!,
	close(Stream).


read_tokens_from(_Stream,-1,[]):-!.

read_tokens_from(Stream,LookAhead,[Punc|Tokens]):-
	is_punctuation_mark(LookAhead),
	!,
	atom_codes(Punc,[LookAhead]),
	get_code(Stream,NextLookAhead),
	read_tokens_from(Stream,NextLookAhead,Tokens).

read_tokens_from(Stream,LookAhead,[Word|Tokens]):-
	is_letter_digit(LookAhead),!,
	read_word(Stream,LookAhead,CharList,NextLookAhead),
	!,
	atom_codes(Word,CharList),
	read_tokens_from(Stream,NextLookAhead,Tokens).

%% ignores everything else
read_tokens_from(Stream,_,Tokens):-
	get_code(Stream,NextLookAhead),
	read_tokens_from(Stream,NextLookAhead,Tokens).





%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
read_word(Stream,LookAhead,[LookAhead|Codes],LastLookAhead):-
	is_letter_digit(LookAhead),
	!,
        get_code(Stream,NextLookAhead),
	read_word(Stream,NextLookAhead,Codes,LastLookAhead).

read_word(_Stream,LookAhead,[],LookAhead).


%% digit
is_letter_digit(Code):-
	Code >= 48, Code =< 57,!.
	
%% uppercase 
is_letter_digit(Code):-
	Code >= 65, Code =<90,!.

%% lowercase 
is_letter_digit(Code):-
	Code >= 97, Code =<122.
	
%%% a puctuation mark
is_punctuation_mark(Code):-
	member(Code,[46,44]).

	
